
#ifndef VFLAGS
#define VFLAGS

#include "vutil.h"

/* Maximum input token size for NEXUS and config files */

#define MAXTOKSIZE 2000

#define MAXPVAL (1.0-(1e-6))  /* If this or greater is entered, */
                              /*   assume that no P-value is wanted. */

/* Basic typedefs */

typedef enum { NOTYPE=0, AUTO, ALL, SILENT, PROTEIN }  SEQTYPE;

typedef enum { TD_NONE=0, TD_BLOCS, TD_SKIP, TD_SITES }  TREAT_DELS;

typedef enum { G_SIMPLE=0, G_BLOCS, G_GROUPS } G_GROUPTYPE;

/* Name is quoted if there are embedded spaces (and truncated) */
/* Rname is unquoted (and untruncated) */

typedef struct { char *name, *rname;
                 int orgoffs; }  SEQNAME;

/* Gname is quoted (truncated) form only. */
/* (PERM_GROUP has unquoted (untruncated) form only.) */

typedef struct { char *gname,*grname;  SEQNAME *sqn;
                 int ns, first,last, gen_poly_only;
                 int dostat, ord; } GROUPLIST;

extern GROUPLIST *grouplist;
extern int ngroups, perm_ngrp;

/* Exportable command-line variables from vsetopts.c */

extern char *seqfile, *outfile, *logfile, *seqfdir;

extern G_GROUPTYPE grouptype;
extern int save_argc, seqfixed_dir;
extern char **save_argv;

/* Basic command-line variables with a non-zero default */

extern char *commchars;
extern char matchchar, gapchar, misschar, numbersep;
extern int numsims, irrupt, jseqlen;
extern int listglobal, listpairs, listperpair;
extern int minlength, minscore, minnpoly;
extern int maxoverlap;
extern SEQTYPE seqtype;
extern TREAT_DELS treat_dels;
extern double maxsimpairpval, maxsimglobpval;
extern double maxkapairpval, maxkaglobpval;
extern int havsimpairpval, havsimglobpval, bcsims;
extern int havkapairpval, havkaglobpval, TabKAPvals;
extern int setkapairpval, setkaglobpval;

extern int innerpairfrags, outerseqfrags, outerpairfrags, outergrpfrags;
extern int allouter, noouter, outersims;
extern int pairplot, altcolor;

#ifdef VSHOW
/* Primarily graphics variables */
/* Vertnumstep returns to defvertnumstep */
/*   unless keepvertscale=TRUE */
extern int fancy_output;
extern int vertnumstep, vertroundstep, defvertnumstep, bcvertscale;
extern double vertminscale, vertmaxscale, vertstepscale;
extern int havminscale, havmaxscale, havstepscale;
extern int keepvertscale, vertskipstep, ylabels_oneside;
extern int addhashrules, skiptext, nocolor;
#endif  /* VSHOW */

/* Used in writing tab files */
extern int havMinDispPvalue;
extern double minDispPvalue;

/* Verbose variables */

extern int gscale, circular, allowindelstrat, havelog;
extern int showunal, havlabels, use_iubnucls, phylip;
extern int nomacfork, no_nexus_dcomm;
extern int dumpfrags, dumptab, dumpdif, dumpjseqs, dumpplot;
extern int dumpexit, use_underscores, commline_shown;
extern int sortnames, sortgfragsbyseq, sortgfragsboth;
extern int annotate, saveseqs, notfound_nonfatal;
extern int show_mmsims, logfragments;
extern int expformat, widecols, AddOffset, truncate_pvals;
extern int showbcpwkapvals, showlog10pvals;
extern unsigned long wstartseed;

/* Lists */
extern STRLIST seq_list, seq_skip_list, filepath, cfgnames;
extern STRBUFF base_range, base_skip_range;

/* Other single-char variables */

extern int nodegenflag, d3flag, simpflag, nosimpflag, mitcodes;
extern int blocsize, homolflag;
extern int ranflag, hold_window, logappend, include_monosites;
extern int showpolyflag, showblast, showfragdata;
extern int ShowNexusToks, showlinesflag, showsites, showtabfrags;

/* Global variables that are not set from the command line */

extern int rshuffled, tabfileflag;

/* Variables saying that a change has been made */

extern int newnumbersep, newstartseed, newlglobal;
extern int commline_sayorg;

/* Called in vswin.c:  return resources if fatal error */
extern int err_clear;
extern void (*err_clear_funp)(void);

/* Prototypes for set_options() and friends */

void set_options (int argc, char *argv[]);
void set_filenames(void);
void no_degens(void);
int set_degens(char *dbuff);
void checkflags(void);
void show_cargcs(const char *bleep, const char *spacestr, int fgap);
void show_commline(const char *bleep, int spaceflag);
int read_sqtype(const char *type);
char *say_sqtype(SEQTYPE type);
char set_vchar(const char *type);
int read_flag (const char *verb, int *vptr, int *newptr);
int read_nflag (const char *verb, int *vptr, int *newptr);
void read_nconfig(const char *p);
int match_showparms(int i1, int i2);
int match_onlylist(int i1, int i2);
int is_onlylist(void);
void kill_testlist (int showflag);
void kill_onlylist (int showflag);
void kill_groups(void);
void init_grouplist(void);
void gscale_comment(void);
char *say_indels(TREAT_DELS tdel);
int anydump(void);
void clear_alldumps(void);
void kill_cfgs(void);

#ifdef VSHOW
void set_base_vars(void);
#endif

#endif /* VFLAGS */


