
#ifndef VUTIL
#define VUTIL

/* Buffer size for warnprintf(), xformout(), getstr(), etc. */

#define SBUFFSIZE 5000

/* If sequence and group names are longer than this, */
/*   they are truncated when quoted. */

#define MAXNAMELEN 200
#define MAXNAMEBUFF (MAXNAMELEN+50)
#define MAXNAMETWO  (2*MAXNAMELEN+200)

typedef void (*VOID_PRINTFUNP)(const char *format, ...);
typedef int (*PRINTFUNP)(const char *format, ...);

/* STRLIST is for a list of strings */
/* STRBUFF is for an expandable string buffer, or */
/*   for a string and two integers in general */

typedef struct { char **s; int n,nx; }  STRLIST, *STRLISTP;

typedef struct { char *s; int n,nx; }  STRBUFF, *STRBUFFP;

extern PRINTFUNP textprintf;
extern void (*textnl)(void);
extern VOID_PRINTFUNP WarnHandler, ErrHandler;

void show_kaparm_errs(PRINTFUNP prfun);

#ifdef VTEXT
int bprintf(const char *control, ...);
void bnl(void);
#define BPRINTF bprintf
#define BNL bnl

#else
#define BPRINTF printf
#define BNL nl
#endif


/* DOS/UNIX dependencies */
#ifdef UNIX

#define READMODE     "r"
#define WRITEMODE    "w"
#define APPENDMODE   "a"
#define PATHSEP      '/'
#define DIRLISTSEP   ':'

#else /* if DOS */

#define READMODE     "rb"
#define WRITEMODE    "wt"
#define APPENDMODE   "at"
#define PATHSEP      '\\'
#define DIRLISTSEP   ';'

typedef enum { TIME_INIT=0, TIME_ELAPSED } TIME_VAR;

void timer (TIME_VAR tvar, int num, char *bleep);

#endif /* UNIX */



/* Simple string functions */

char *makeuc(char *str);
char *makelc(char *str);
int stricmp(const char *p, const char *q);
int striucmp(const char *p, const char *q);
int strincmp(const char *p, const char *q, size_t n);
int strmatch(const char *p, const char *mat);
int strlenx(const char *p);
char *bool(int flag), *boolon(int flag);
char *parse_unquote (char *p, char **nextp, int *openflagp);
char *parse_unquote_sep (char *p, char **nextp, char *sep, int *openfp);
char *doquote (char *buff, const char *str);
int quotesclen (char *str);
char *gettag(char *rname, char **rdtagptr, char **rdshowptr, char numchar);

int string_sort (const void *p, const void *q);
int sortint (const void *p, const void *q);
int sortdub (const void *p, const void *q);

char *commll(char *cbuff, unsigned long uu);
char *numsay (char *nbuff, long n, char *bleep, int capflag);
char *numsayc (char *nbuff, long n, char *bleep, int capflag);
char *getstr(const char *control, ...);


/* Display and parsing functions */

typedef struct { char *drive, *path, *name, *ext; }  PARTFN;

char *makefnam (const char *rawfn, const char *tplate);
PARTFN partfn (const char *source);
char *makefmpf (PARTFN pf);
int checkfname(const char *filename);
void freepf (PARTFN pf);


/* Simple file-oriented utilities */

FILE *fxopenwrite(const char *path, const char *mode);
int fxclose(FILE *fp, const char *fname);
FILE *fxopenread(const char *fname,
  char **openname, char **seqpath, int *fpp, int tabflag);
int isextent(const char *p, const char *ext);
char *say_type (const char *fname);
int type_msg (char *vbuff, const char *bleep, const char *readfile);
char *shortname(char *sbuff, const char *fname);
int addprintf (STRBUFFP ap, int maxlen, const char *control, ...);
void showpath (STRBUFFP ap);


/* Argument formatting and specialized print functions */

void nl(void);
void warnprintf(const char *control, ...);
void exitprintf(const char *control, ...);
void xformout (const char *control, ...);
void set_textprintf(PRINTFUNP prfun, void (*prnl)(void));
void reset_textprintf(void);
#ifndef UNIX
void presskey(void);
#endif

/* Token-printing package with no word-wrap within tokens */
/* Basic functions */

void ftoklistf (const char *bleep, char **vars,int num,
   int nrgap,const char *sep,int fgap, int addflags);
void toklist (const char *bleep, char **vars,int num);

extern int tok_linesize, dumpflag, notimer;    /* Line size */

/* Functions with token structures */

#define TOK_NONE         0
#define TOK_FREE         1
#define TOK_FREE_ALL   TOK_FREE
#define TOK_QUOTE        2
#define TOK_VOLATILE     4
#define TOK_ENDPERIOD    8
#define TOK_GROUPS      16

typedef struct { char *str, *frstr;  int fflag; }  TOKSTRUC;

typedef struct { TOKSTRUC *tok;  int n,nx; } TOKLIST, *TOKLISTP;

char *addtokl (TOKLISTP tokl, char *str, int addflags);

void ftoklistex (const char *bleep, TOKLISTP tokl,
   int nrgap,const char *sep,int fgap, int addflags);

int tokl_string_sort (TOKLIST dd);

/* Memory allocation functions with error checks */

extern int numrealloc;
extern unsigned long totalloc, totrealloc;

void show_alloc (VOID_PRINTFUNP prfun);
void show_alloc_sofar (VOID_PRINTFUNP prfun);
void *mxalloc (size_t size);
void *cxalloc (size_t nitems, size_t size);
char *strxdup(const char *p);
char *strxndup(const char *p, size_t n);
void *rexalloc(void *p, size_t newsize);

void *rexxalloc(void *p, size_t ssize, size_t oldnum, size_t newnum);

void *addlist(void *ss, size_t size, void **slist,int *nsp,int *nxsp);
char *addstr(STRLISTP ap, const char *str);
char *check_addstr(STRLISTP ap, char *str);
char *check_addseq(STRLISTP ap, char *str);
char *unspace_str(char *str);
void unspace_slist1(char **str, int n);
void unspace_slist2(STRLIST a);
void kill_strlist (STRLISTP ap);
void kill_nstr (char ***listp, int n);
void kill_strbuff (STRBUFFP ap);
int parse_addpath(STRLISTP ap, char *str);


/* Package for overlap control */

extern int showoverlap, maxoverlap;
int checkoverlap(int op, int start, int fin);


/* Random number package */

int nrand(int m);
unsigned long setrand(unsigned long initseed);
unsigned long dispseed(void);

#endif /* VUTIL */

